@@ -70,7 +70,7 @@ static dnsheader_t *decode_header(void *packet, int len)
     p = (unsigned short int *) x->packet;
 
     x->id      = ntohs(p[0]);
-    x->u.flags = ntohs(p[1]);
+    x->u.flags = p[1]; // Do NOT put this in host order!
     x->qdcount = ntohs(p[2]);
     x->ancount = ntohs(p[3]);
     x->nscount = ntohs(p[4]);
@@ -118,28 +118,15 @@ static int get_objectname(unsigned char *msg, unsigned char **here,
     while ((len = **here) != 0) {
 	*here += 1;
 
-	if ((len & 0xC0) == 0xC0) {
-	    int	offset;
+	if (len & 0xC0) {
+	    unsigned offset;
 	    unsigned char *p;
 
-	    offset = len & 0x3F;
-	    if (offset == 0x00) {
+	    offset = ((len & ~0xc0) << 8) + **here;
+	    p = &msg[offset];
+	    k = get_objectname(msg, &p, string, k);
 
-	        /*
-		 * Just guessing - this is not documented
-		 * in RFC 1035.
-		 */
-
-		offset = **here;
-		p = &msg[offset];
-		k = get_objectname(msg, &p, string, k);
-
-		break;
-	    }
-	    else {
-		p = &msg[offset];
-		k = get_objectname(msg, &p, string, k);
-	    }
+	    break;
 	}
 	else if (len < 64) {
 	    for (i=0; i < len; i++) {
@@ -161,6 +148,7 @@ static unsigned char *read_record(dnsheader_t *x, rr_t *y,
 				  unsigned char *here, int question)
 {
     int	k, len;
+    unsigned short int conv;
 
     /*
      * Read the name of the resource ...
@@ -173,16 +161,16 @@ static unsigned char *read_record(dnsheader_t *x, rr_t *y,
      * ... the type of data ...
      */
 
-    memcpy(&y->type, here, sizeof(unsigned short int));
-    y->type = ntohs(y->type);
+    memcpy(&conv, here, sizeof(unsigned short int));
+    y->type = ntohs(conv);
     here += 2;
 
     /*
      * ... and the class type.
      */
 
-    memcpy(&y->class, here, sizeof(unsigned short int));
-    y->class = ntohs(y->class);
+    memcpy(&conv, here, sizeof(unsigned short int));
+    y->class = ntohs(conv);
     here += 2;
 
     /*
@@ -239,7 +227,7 @@ int dump_dnspacket(char *type, unsigned char *packet, int len)
 	     x->u.b.want_recursion, x->u.b.recursion_avail,
 	     x->u.b.truncated,
 	     x->u.b.rcode,
-	     x->u.flags);
+	     ntohs(x->u.flags));
 
     fprintf(stderr, "qd= %u\n", x->qdcount);
     x->here = read_record(x, &y, x->here, 1);
@@ -314,7 +302,7 @@ unsigned char *parse_query(rr_t *y, unsigned char *msg, int len)
 	return (0);
     }
 
-    y->flags.flags = ntohs(((short int *) msg)[1]);
+    y->flags.flags = ((short int *) msg)[1];
 
     here = &msg[PACKET_DATABEGIN];
     k = get_objectname(msg, &here, y->name, 0);
