@@ -589,7 +589,8 @@ static int end_assembly(dnsheader_t *x)
 /*  x->u.b.authorative = 0; */
 
     pkt = (unsigned short int *) x->packet;
-    pkt[1] = x->u.flags;
+    // replaced by guido:
+    pkt[1] = htons(x->u.flags);
     pkt[2] = htons(x->qdcount);
     pkt[3] = htons(x->ancount);
     pkt[4] = htons(x->nscount);
